<?php

#########################################################
#                                                       #
#  Novalnet payment method class                        #
#  This module is used for real time processing of      #
#  Novalnet transaction of customers.                   #
#                                                       #
#  Copyright (c) Novalnet AG                            #
#                                                       #
#  Released under the GNU General Public License        #
#  This free contribution made by request.              #
#  If you have found this script usefull a small        #
#  recommendation as well as a comment on merchant form #
#  would be greatly appreciated.                        #
#                                                       #
#  Script : uc_novalnet.inc                             #
#                                                       #
#########################################################

  include_once(drupal_get_path('module', 'uc_novalnet') . '/uc_novalnet_css_link.inc'); // for custom css 
  
  /**
   * get the payment name
   *
   * @param  none
   * @return array
   */
  function uc_novalnet_payment_methods() {
    return array(
      'novalnet_prepayment'   => t('Prepayment'),
      'novalnet_invoice'      => t('Invoice'),
      'novalnet_cc'           => t('Credit Card'),
      'novalnet_paypal'       => t('PayPal'),
      'novalnet_ideal'        => t('iDEAL'),
       'novalnet_sepa'        => t('Direct Debit SEPA'),
      'novalnet_instantbank'  => t('Instant Bank Transfer'),
      'novalnet_telephone'    => t('Telephone Payment'),

    );
  }
  
  /**
   * get the payment key
   *
   * @param  none
   * @return array
   */
  function uc_novalnet_payment_key() {
    return array(
      'novalnet_cc'           => 6,
      'novalnet_telephone'    => 18,
      'novalnet_prepayment'   => 27,
      'novalnet_invoice'      => 27,
      'novalnet_instantbank'  => 33,
      'novalnet_paypal'       => 34,
      'novalnet_ideal'        => 49,
      'novalnet_sepa'         => 37,
    );
  }
  
  /**
   * set the payment gateway url's
   *
   * @param  $order_payment
   * @return string
   */
  function uc_novalnet_redirect_urls($order_payment) {
    global $is_https;
    $url = array(
      'novalnet_prepayment'   => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' : 'http://payport.novalnet.de/paygate.jsp',
      'novalnet_invoice'      => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' : 'http://payport.novalnet.de/paygate.jsp',
      'novalnet_cc'           => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' : 'http://payport.novalnet.de/paygate.jsp',
      'novalnet_cc3d'         => ($is_https)? 'https://payport.novalnet.de/global_pci_payport' : 'http://payport.novalnet.de/global_pci_payport',
      'novalnet_sepa'         => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' :'http://payport.novalnet.de/paygate.jsp',
      'novalnet_paypal'       => ($is_https)? 'https://payport.novalnet.de/paypal_payport' : 'http://payport.novalnet.de/paypal_payport',
      'novalnet_ideal'        => ($is_https)? 'https://payport.novalnet.de/online_transfer_payport' : 'http://payport.novalnet.de/online_transfer_payport',
      'novalnet_instantbank'  => ($is_https)? 'https://payport.novalnet.de/online_transfer_payport' : 'http://payport.novalnet.de/online_transfer_payport',
      'novalnet_telephone'    => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' : 'http://payport.novalnet.de/paygate.jsp',
      'novalnet_postbackcall' => ($is_https)? 'https://payport.novalnet.de/paygate.jsp' : 'http://payport.novalnet.de/paygate.jsp',
      'novalnet_cc_iframe'    => ($is_https)? 'https://payport.novalnet.de/direct_form.jsp' : 'http://payport.novalnet.de/direct_form.jsp',
      'novalnet_sepa_iframe'  => ($is_https)? 'https://payport.novalnet.de/direct_form_sepa.jsp' : 'http://payport.novalnet.de/direct_form_sepa.jsp',
    );
    return trim($url[$order_payment]);
  }
  
   /**
   * Validate configuration fields
   *
   * @param  $element
   * @param  $form_state
   * @return none
   */
   
  function uc_is_digit($element, &$form_state)  {
    if (trim($element['#value']) != '') {
      if (!preg_match("/^[0-9]+$/", trim($element['#value']))) {
        $errormsg = $element['#title'] . ' ' . t('not valid');
          form_error($element, $errormsg);
    }
    }
  }
  
  /**
    * Validate duedate
    *
    * @param  $element
    * @param  $form_state
    * @return none
    */
    
  function uc_due_date_validation($element, &$form_state) {

    if (trim($element['#value']) != '') {
     if ($element['#name'] == 'novalnet_sepa_duedate' && $form_state['input']['novalnet_sepa_paymenttype'] == 'DIRECT_DEBIT_SEPA' && ($element['#value'] < 7 ||!preg_match("/^[0-9]+$/", trim($element['#value'])))) {
        $errormsg = $element['#title'] . ' ' . t('not valid');
        form_error($element, $errormsg);
      }
    }
  }
  
   /**
   * Validate additional configuration fields
   *
   * @param  $element
   * @param  $form_state
   * @return none
   */
  function uc_is_digit_additional($element, &$form_state)  {
    $payment = $form_state['build_info']['args'][0];
    $values  = $form_state['values'];
    if (isset($values[$payment . '_manualamount']) && trim($values[$payment . '_manualamount'])) {
      if (trim($element['#value']) != '') {
        if (!preg_match("/^[0-9]+$/", trim($element['#value']))) {
          $errormsg = $element['#title'] . ' ' . t('not valid');
          form_error($element, $errormsg);
        }
      }
    }
  }

  /**
   * set the Admin side configuration fields
   *
   * @param  $payment_select
   * @return array
   */
  function uc_novalnet_backend_config($payment_select) {
    $payment_pasword      = array('novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank');
    $payment_manuallimit  = array('novalnet_cc','novalnet_sepa');
    $fraud_prevention     = array('novalnet_cc','novalnet_sepa','novalnet_invoice');
    $form[$payment_select . '_testmode'] = array(
      '#type'          => 'select',
      '#title'         => t('Enable Test Mode'),
      '#options'       => array(1 => t('True'), 0 => t('False')),
      '#default_value' => variable_get($payment_select . '_testmode', 0),

    );
    $form[$payment_select . '_nnlogo'] = array(
      '#type'          => 'select',
      '#title'         => t('Enable Novalnet Logo'),
      '#description'   => t('To display Novalnet logo in front end'),
      '#options'       => array(1 => t('True'), 0 => t('False')),
      '#default_value' => variable_get($payment_select . '_nnlogo', 1),
    );
    $form[$payment_select . '_paymentlogo'] = array(
      '#type'          => 'select',
      '#title'         => t('Enable Payment logo'),
      '#description'   => t('To display Payment logo in front end'),
      '#options'       => array(1 => t('True'), 0 => t('False')),
      '#default_value' => variable_get($payment_select . '_paymentlogo', 1),
    );
    $form[$payment_select . '_vendorid'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Novalnet Merchant ID'),
      '#description'   => t('Enter your Novalnet Merchant ID'),
      '#default_value' => variable_get($payment_select . '_vendorid', ''),
      '#required'      => TRUE,
      '#element_validate' => array('uc_novalnet_config_validation'),
      );
    $form[$payment_select . '_authcode'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Novalnet Merchant Authorisation code'),
      '#description'   => t('Enter your Novalnet Merchant Authorisation code'),
      '#default_value' => variable_get($payment_select . '_authcode', ''),
      '#required'      => TRUE,
      );
    $form[$payment_select . '_productid'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Novalnet Product ID'),
      '#description'   => t('Enter your Novalnet Product ID'),
      '#default_value' => variable_get($payment_select . '_productid', ''),
      '#required'      => TRUE,
      '#element_validate' => array('uc_novalnet_config_validation'),

      );
    $form[$payment_select . '_tariffid'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Novalnet Tariff ID'),
      '#description'   => t('Enter your Novalnet Tariff ID'),
      '#default_value' => variable_get($payment_select . '_tariffid', ''),
      '#required'      => TRUE,
      '#element_validate' => array('uc_novalnet_config_validation'),
      );
      
     if (in_array($payment_select, $fraud_prevention)) {        
      $options = array('false'    => t('Not active'), 
                       'callback' => t('Callback (Landline & mobile)'), 
                       'sms'      => t('SMS (Mobile only)'), 
                       'email'    => t('Email Reply')); 
      $form['fraud_prevention'] = array(
          '#type'             => 'fieldset',
          '#title'            => t('Novalnet').' '.t('Fraudprevention'). ' ' .t('configuration'),
          '#collapsible'      => TRUE,
          '#collapsed'        => TRUE,
          '#states'           => array(
                            'invisible' => array(
                             ':input[name="novalnet_cc_activatecc3d"]' => array('checked' => TRUE),
                              )));              
      $form['fraud_prevention'][$payment_select . '_fraud_prevention'] = array(
        '#type'          => 'radios',
        '#title'         => t('Fraudprevention through PIN by Callback/SMS/E-Mail'),
        '#default_value' => variable_get($payment_select . '_fraud_prevention', 'false'),
        '#options'       => $options,
        '#description'   => t('When activated by PIN Callback / SMS / E-Mail the customer to enter their phone / mobile number / E-Mail requested. By phone or SMS, the customer receives a PIN from Novalnet AG, which must enter before ordering. If the PIN is valid, the payment process has been completed successfully, otherwise the customer will be prompted again to enter the PIN. This service is only available for customers from specified countries')
      ); 
      $form['fraud_prevention'][$payment_select . '_fraud_prevention_amount'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Minimum Amount Limit for Callback in cents'),
        '#default_value' => variable_get($payment_select . '_fraud_prevention_amount', ''),
        '#element_validate' => array('uc_novalnet_config_validation'),
        '#states'           => array('invisible' =>
                                    array('input[type="radio"]' => array('value' => 'false'))),
      );
      }
                    
      $form[$payment_select . '_referer_id'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Referrer ID'),
      '#description'   => t('Referrer ID of the partner at Novalnet, who referred you (only numbers allowed)'),
      '#default_value' => variable_get($payment_select . '_referer_id', ''),
      '#element_validate' => array('uc_novalnet_config_validation'),
      );
      $cat_option = uc_order_status_options_list();
      if ($payment_select == 'novalnet_prepayment' || $payment_select == 'novalnet_invoice' || $payment_select == 'novalnet_paypal'){
      $form[$payment_select . '_before_order_status'] = array(
      '#type'          => 'select',
      '#title'         => t('Order Status Before Payment'),
      '#options'       => $cat_option,
      '#default_value' => variable_get($payment_select . '_before_order_status', ''),
      );
      $form[$payment_select . '_after_order_status'] = array(
      '#type'          => 'select',
      '#title'         => t('Order Status After Successful Payment'),
      '#options'       => $cat_option,
      '#default_value' => variable_get($payment_select . '_after_order_status', ''),
      );
      }
      else {
      $form[$payment_select . '_order_status'] = array(
      '#type'          => 'select',
      '#title'         => t('Order Status After Successful Payment'),
      '#options'       => $cat_option,
      '#default_value' => variable_get($payment_select . '_order_status', ''),
      );
        
     }
    if (in_array($payment_select, $payment_manuallimit)) {
      $form[$payment_select . '_manualamount'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Manual checking of order, above amount in cents (Note: this is a onhold booking, needs your manual verification and activation)'),
        '#description'   => t('All the orders above this amount will be set on hold by Novalnet and only after your manual verifcation and confirmation at Novalnet the booking will be done'),
        '#default_value' => variable_get($payment_select . '_manualamount', ''),
        '#required'      => FALSE,
        '#element_validate' => array('uc_novalnet_config_validation'),
        );
      $form[$payment_select . '_secondproduct'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Second Product ID for manual check condition'),
        '#description'   => t('Second Product ID in Novalnet to use the manual check condition'),
        '#default_value' => variable_get($payment_select . '_secondproduct', ''),
        '#element_validate' => array('uc_novalnet_config_validation'),
        );
      $form[$payment_select . '_secondtariff'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Second Tariff ID for manual check condition'),
        '#description'   => t('Second Tariff ID in Novalnet to use the manual check condition'),
        '#default_value' => variable_get($payment_select . '_secondtariff', ''),
        '#element_validate' => array('uc_novalnet_config_validation'),
        );
    }
    if ($payment_select == 'novalnet_cc') {
        
      $form[$payment_select . '_refill'] = array(
        '#type'          => 'checkbox',
        '#title'         => t('Auto refill the payment data entered in payment page'),
        '#default_value' => variable_get($payment_select . '_refill', ''),
        '#required'      => FALSE,
        );
        
      $form[$payment_select . '_activatecc3d'] = array(
        '#type'          => 'checkbox',
        '#title'         => t('3D Secure(note : this has to be set up at Novalnet first. Please contact support@novalnet.de, in case you wish this.)'),
        '#description'         => t('(Please note that this procedure has a low acceptance among end customers.) As soon as 3D-Secure is activated for credit cards, the bank prompts the end customer for a password, to prevent credit card abuse. This can serve as a proof, that the customer is actually the owner of the credit card.'),
        '#default_value' => variable_get($payment_select . '_activatecc3d', ''),
        '#required'      => FALSE,
        '#element_validate' => array('uc_novalnet_config_validation'),
        );
         /*Payment access key enabled for CC3d secure*/
         $form[$payment_select . '_accesskey'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Novalnet Payment Access Key'),
        '#default_value' => variable_get($payment_select . '_accesskey', ''),
        '#description'    => t('Enter your Novalnet payment access key'),
        '#states'         => array(
                            'visible' => array(
                             ':input[name="novalnet_cc_activatecc3d"]' => array('checked' => TRUE),
                              ), ),
      );   
       $form[$payment_select . '_amexlogo'] = array(
      '#type'          => 'select',
      '#title'         => t('Enable AMEX logo'),
      '#description'   => t('To display AMEX logo in front end'),
      '#options'       => array(1 => t('True'), 0 => t('False')),
      '#default_value' => variable_get($payment_select . '_amexlogo', 0),
    ); 
    }
    if (in_array($payment_select, $payment_pasword)) {
      $form[$payment_select . '_accesskey'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Novalnet Payment access key'),
        '#description'   => t('Enter your Novalnet payment access key'),
        '#default_value' => variable_get($payment_select . '_accesskey', ''),
        '#required'      => TRUE,
       );      
    }

    if ($payment_select == 'novalnet_sepa') {
      $sepatype = array('DIRECT_DEBIT_SEPA' => t('Direct Debit SEPA'));
      $description = t('Direct Debit SEPA') . ' : ' . t('Please note the transaction will be completed immediately with out mandate.'). '<br>';

      $form[$payment_select . '_duedate'] = array(
        '#type'          => 'textfield',
        '#title'         => t('SEPA Payment duration in days'),
        '#default_value' => variable_get($payment_select . '_duedate', ''),
        '#description'    => t('Enter the Due date in days, it should be greater than 6. If you leave as empty means default value will be considered as 7 days.'),
        '#element_validate' => array('uc_novalnet_config_validation'),
      );
      $form[$payment_select . '_refill'] = array(
        '#type'          => 'checkbox',
        '#title'         => t('Auto refill the payment data entered in payment page'),
        '#default_value' => variable_get($payment_select . '_refill', ''),
        '#required'      => FALSE,
        );
    }
    if ($payment_select == 'novalnet_invoice') {
      $form[$payment_select . '_duedate'] = array(
        '#type'          => 'textfield',
        '#title'         => t('Payment period in days'),
        '#default_value' => variable_get($payment_select . '_duedate', ''),
        '#description'   => t('Enter the due date in days, it should be greater than 6. If you leave as empty or less than 7 means default value will be considered as 14 days.'),
        '#required'      => FALSE,
        '#element_validate' => array('uc_is_digit'),
        );
    }
    if ($payment_select == 'novalnet_paypal') {
      $form[$payment_select . '_apiuser'] = array(
        '#type'          => 'textfield',
        '#title'         => t('PayPal API User Name'),
        '#description'   => t('Please enter your PayPal API username'),
        '#default_value' => variable_get($payment_select . '_apiuser', ''),
        '#required'      => TRUE,
        );
      $form[$payment_select . '_apipassword'] = array(
        '#type'          => 'textfield',
        '#title'         => t('PayPal API Password'),
        '#description'   => t('Please enter your PayPal API password'),
        '#default_value' => variable_get($payment_select . '_apipassword', ''),
        '#required'      => TRUE,
        );
      $form[$payment_select . '_apisignature'] = array(
        '#type'          => 'textfield',
        '#title'         => t('PayPal API Signature'),
        '#description'   => t('Please enter your PayPal API signature'),
        '#default_value' => variable_get($payment_select . '_apisignature', ''),
        '#required'      => TRUE,
        );
    }
    $form[$payment_select .'_reference1'] = array(
    '#type'        =>  'textfield',
    '#title'       =>  t('Transaction reference 1'),
    '#default_value'  => variable_get($payment_select .'_reference1', ''),
    '#description' => t('This will appear in the transactions details / account statement'),
    '#element_validate' => array('uc_novalnet_config_validation'),
    );
    $form[$payment_select .'_reference2'] = array(
    '#type'        =>  'textfield',
    '#title'       =>  t('Transaction reference 2'),
    '#default_value'  => variable_get($payment_select .'_reference2', ''),
    '#description' => t('This will appear in the transactions details / account statement'),
    '#element_validate' => array('uc_novalnet_config_validation'),
    );
  return $form;
  }
  
  /*back end validation*/
  function uc_novalnet_config_validation($element, &$form_state) {
    $form_state['values'] = array_map('strip_tags', $form_state['values']);
    $form_state['values'] = array_map('trim', $form_state['values']);
    $values = $form_state['values'];
    $ele = explode("_", $element['#name']);
    $ele = $ele[0].'_'.$ele[1];
    
    if (!is_numeric($values[$ele.'_vendorid'])) {
        form_set_error($ele.'_vendorid', t('Novalnet Merchant ID') . ' ' . t('not valid'));
    }
    if (!is_numeric($values[$ele.'_productid'])) { 
        form_set_error($ele.'_productid', t('Novalnet Product ID') . ' ' . t('not valid'));
    }
    if (!is_numeric($values[$ele.'_tariffid'])) {
        form_set_error($ele.'_tariffid', t('Novalnet Tariff ID') . ' ' . t('not valid'));
    }
    if (!empty($values[$ele.'_referer_id']) && !is_numeric($values[$ele.'_referer_id'])) {
        form_set_error($ele.'_referrerid', t('Referrer ID') . ' ' . t('not valid'));
    }
    if(isset($values['novalnet_cc_activatecc3d']) && $values['novalnet_cc_activatecc3d']){
        ($values['novalnet_cc_accesskey'] == '') ? form_set_error('novalnet_cc_accesskey', t('Payment Access Key'). ' ' . t('not valid')) : '' ;
    } 
    if(isset($values[$ele.'_manualamount'])){
       if ($values[$ele.'_manualamount'] != '' ){
        $error = false;
        if(!is_numeric($values[$ele.'_manualamount'])) {
        $error = true;
        form_set_error($ele.'_manualamount', t('Manual limit amount / Product-ID2 / Tariff-ID2 is not valid'));
        }
        else{
        if(!is_numeric($values[$ele.'_secondproduct'])){
        ($error) ? form_set_error($ele.'_secondproduct') : form_set_error($ele.'_secondproduct', t('Manual limit amount / Product-ID2 / Tariff-ID2 is not valid'));
        $error = true;
        }
        if(!is_numeric($values[$ele.'_secondtariff'])){
        ($error) ? form_set_error($ele.'_secondtariff') : form_set_error($ele.'_secondtariff', t('Manual limit amount / Product-ID2 / Tariff-ID2 is not valid'));
        }
       }
      }
    }
    $fraud_prevention = array('novalnet_cc','novalnet_sepa','novalnet_invoice');
    if(in_array($ele, $fraud_prevention) && ($values[$ele.'_fraud_prevention'] != 'false') && !empty($values[$ele.'_fraud_prevention_amount']) && !is_numeric($values[$ele.'_fraud_prevention_amount'])){
        fform_set_error($ele.'_fraud_prevention_amount', t('Minimum Amount Limit for Callback in cents') . ' ' . t('not valid'));
    }
    
    if(isset($values['novalnet_sepa_duedate'])){
    if ($values['novalnet_sepa_duedate'] != '' && (!is_numeric($values['novalnet_sepa_duedate']) || (int) $values['novalnet_sepa_duedate'] < 7 )) {
        form_set_error($ele.'_duedate', t('SEPA Due date is not valid'));
    }
    }
}
function uc_novalnet_fraudprevention_data_validation($element, &$form_state) {
    $values = $form_state['values']['panes']['payment']['details'];
    $ele = explode("_", end($element['#array_parents'])); 
    $ele = $ele[0].'_'.$ele[1]; 
    if(empty($values[$ele.'_pin']) && empty($values[$ele.'_forget_pin'])) { 
        form_set_error($ele.'_pin', t('PIN you have entered is incorrect or empty!'));
    }
    if(isset($values[$ele.'_pin']) && !($values[$ele.'_forget_pin']) && preg_match('/[#%\^<>@$=*!]/', $values[$ele.'_pin'])) { 
        form_set_error($ele.'_pin', t('PIN you have entered is incorrect or empty!'));
    }
}

  /**
   * set the payment description
   *
   * @param  $payment_select
   * @return string
   */
  function uc_novalnet_payment_desc($selected_payment) {
    $payment_description = array(
      'novalnet_prepayment'  => t('The bank details will be emailed to you soon after the completion of checkout process.'),
      'novalnet_invoice'     => t('The bank details will be emailed to you soon after the completion of checkout process.'),
      'novalnet_cc'          => t('The amount will be booked immediately from your credit card when you submit the order.'),
      'novalnet_paypal'      => t('You will be redirected to Novalnet AG website when you place the order.'),
      'novalnet_ideal'       => t('You will be redirected to Novalnet AG website when you place the order.'),
      'novalnet_instantbank' => t('You will be redirected to Novalnet AG website when you place the order.'),
      'novalnet_sepa'        => t('Your account will be debited upon delivery of goods.'),
      'novalnet_telephone'   => t('Your amount will be added in your telephone bill when you place the order.'),
    );

    return $payment_description[$selected_payment];
  }
  /**
   * set the payment testmode
   *
   * @param  $payment_select
   * @return string
   */
  function uc_novalnet_payment_testmode($selected_payment) {
        return '<div style =color:red;>' . t('Please Note: This transaction will run on TEST MODE and the amount will not be charged') . '</div>';
  }
   /**
   * Set the title of the payment method
   *
   * @param  $method_id string
   * @param  $nn_logo_status boolean
   * @param  $payment_logo_status boolean
   * @return string
   */
    function uc_novalnet_payment_title($method_id, $nn_logo_status, $payment_logo_status){
   //images from local;
    $get_path = drupal_get_path('module', 'uc_novalnet');
    $get_path = url('',array('relative'=>true )) . $get_path;
    $nn_logo_url    = $get_path.'/img/NN_Logo_T.png';
    $payment_name    = uc_novalnet_payment_methods();
    $current_payment = $payment_name[$method_id];
    
    $is_amex = variable_get('novalnet_cc_amexlogo') == 1 ? $get_path.'/img/amex.png' : $get_path.'/img/creditcard_small.png';
    $payment_logos   = array(
      'novalnet_prepayment'  => $get_path.'/img/Vorkasse.png',
      'novalnet_invoice'     => $get_path.'/img/Rechnungskauf.png',
      'novalnet_cc'          => $is_amex,
      'novalnet_cc_amex'     => $get_path.'/img/amex_logo.png',
      'novalnet_paypal'      => $get_path.'/img/Paypal.png',
      'novalnet_ideal'       => $get_path.'/img/iDeal.png',
      'novalnet_instantbank' => $get_path.'/img/Sofort.png',
      'novalnet_sepa'        => $get_path.'/img/sepa.png',
      'novalnet_telephone'   => $get_path.'/img/Telefonpayment.png',
    );
    $get_novalnet_logo = '<img src="' . $nn_logo_url . '" alt="Novalnet AG" title="Novalnet AG" height="25px"/>';
    $nn_logo = l($get_novalnet_logo, t('http://www.novalnet.com'), array('attributes'=>array('target'=>'_blank'), 'html'=>true));
    $img_logo_url = $payment_logos[$method_id];
    $get_pay_logo = '<img src="' . $img_logo_url . '" alt="'.t($current_payment).'" title="'.t($current_payment).'" />';

    $nn_pay_logo  =l($get_pay_logo, t('http://www.novalnet.com'), array('attributes'=>array('target'=>'_blank'), 'html'=>true));
    $nn_logo      = (($nn_logo_status == 1) && isset($nn_logo_status)? $nn_logo : '');
    $nn_pay_logo  = (($payment_logo_status == 1) && isset($payment_logo_status)? $nn_pay_logo : '');
    $title        = $nn_logo . t($current_payment) . $nn_pay_logo;

  return $title;
  }
   /**
   * Get the basic parameters of the payment method
   *
   * @param  $data array
   * @param  $payment_method string
   * @return array
   */
  function uc_novalnet_basic_params(&$data, $payment_method) {
    $data['vendor_id']       = trim(variable_get($payment_method . '_vendorid',  ''));
    $data['product_id']      = trim(variable_get($payment_method . '_productid',  ''));
    $data['tariff_id']       = trim(variable_get($payment_method . '_tariffid',  ''));
    $data['vendor_authcode'] = trim(variable_get($payment_method . '_authcode',  ''));
    foreach ($data as $values) {
      if (empty($values)) {
        $nnencodeerror       = check_plain(t('Basic parameter not valid'));
        drupal_set_message(check_plain($nnencodeerror), 'warning');
        watchdog('uc_novalnet', $payment_method, array('@error' => $nnencodeerror), WATCHDOG_ERROR);
        drupal_goto('cart/checkout');
        exit;
      }
    }
    $data['test_mode']       = variable_get($payment_method . '_testmode', 0);
    return True;
  }
  /**
   * Get the additional parameters of the payment method
   *
   * @param  $data array
   * @param  $payment_method string
   * @return array
   */
  function uc_novalnet_getsecondparam(&$data, $payment_select) {
    $manual_limit = intval(trim(variable_get($payment_select . '_manualamount', '')));
    if ($manual_limit && is_numeric($manual_limit)) {
      $id2 = array();
      $id2['product_id2'] = trim(variable_get($payment_select . '_secondproduct', ''));
      $id2['tariff_id2']  = trim(variable_get($payment_select . '_secondtariff', ''));
      if (empty($id2['product_id2']) || empty($id2['tariff_id2'])) {
        drupal_set_message(check_plain(t('Manual limit amount / Product-ID2 / Tariff-ID2 is not valid')),  'warning');
        drupal_goto('cart/checkout');
        exit;
      } elseif ($data['amount'] >= $manual_limit && $manual_limit > 0 ) {
        $data['product_id'] = $id2['product_id2'];
        $data['tariff_id']  = $id2['tariff_id2'];
      }
    }
  }
  /**
   * Set the data array for non redirect type of payment methods
   *
   * @param  $data array
   * @return array
   */
  function uc_novalnet_rebuildparam(&$data) {
    $data['vendor']    = $data['vendor_id'];
    $data['product']   = $data['product_id'];
    $data['tariff']    = $data['tariff_id'];
    $data['auth_code'] = $data['vendor_authcode'];
    unset($data['vendor_id']);
    unset($data['product_id']);
    unset($data['tariff_id']);
    unset($data['vendor_authcode']);
  }

  function uc_novalnet_fraudprevention_data(&$data, $payment) {
    if (variable_get($payment.'_fraud_prevention', '') == 'callback'){
        $data['pin_by_callback'] = '1';
        $data['tel'] = $_SESSION['novalnet'][$payment]['tel'];
    }             
    if (variable_get($payment.'_fraud_prevention', '') == 'sms') {
        $data['pin_by_sms'] ='1';
        $data['mobile'] = $_SESSION['novalnet'][$payment]['tel'];
        $data['tel'] = '';
    }
    if (variable_get($payment.'_fraud_prevention', '') == 'email') {
        $data['reply_email_check'] ='1';
        $data['email'] = $_SESSION['novalnet'][$payment]['email'];
    }
    return $data;
  }
  
   /**
   * Set the amount in cents
   *
   * @param  $data array
   * @param  $orderparam array
   * @return array
   */
  function uc_novalnet_getamount(&$data, $orderparam) {
    $data['amount'] = uc_novalnet_orderamt2novalnetamt($orderparam);
    if ($data['amount'] != '') {
      if (!preg_match("/^[0-9]+$/", trim($data['amount']))) {
        $nnencodeerror       = check_plain(t('Amount value is not valid'));
        drupal_set_message(check_plain($nnencodeerror), 'warning');
        drupal_goto('cart/checkout');
        exit;
      }
  }
  }
  /**
   * Get the amount in cents to send server
   *
   * @param  $orderparam array
   * @return double
   */
  function uc_novalnet_orderamt2novalnetamt($orderparam) {
    $total_amount = round($orderparam->order_total, 2);
    $amount       = sprintf('%.2f',  $total_amount);
    return $amount*100;
  }
  /**
   * Set additional order parameters
   *
   * @param  $data array
   * @param  $language array
   * @param  $orderparam array
   * @return array
   */
  function uc_novalnet_default_param_list(&$data, $language, $orderparam) {
    $billing_country          = uc_get_country_data(array('country_id' => $orderparam->billing_country));
    $country                  = $billing_country[0]['country_iso_code_2'];
    $customer_no              = (($orderparam->uid) > 0 ) ? $orderparam->uid : 'guest';
    $orderuid                 = (($orderparam->uid) > 0 ) ? $orderparam->uid : 'NULL';
    $ipaddr                   = get_realip();
    $data['currency']         = variable_get('uc_currency_code',  'USD');
    $data['first_name']       = trim($orderparam->billing_first_name);
    $data['last_name']        = trim($orderparam->billing_last_name);
    $data['gender']           = 'u';
    $data['email']            = trim($orderparam->primary_email);
    $data['street']           = $orderparam->billing_street1;
    if (isset($orderparam->billing_street2) && !empty($orderparam->billing_street2)) {
      $data['street'] .= ' ' . $orderparam->billing_street2;
    }
    $data['search_in_street'] = 1;
    $data['city']             = $orderparam->billing_city;
    $data['zip']              = $orderparam->billing_postal_code;
    $data['tel']              = trim($orderparam->billing_phone);
    $data['lang']             = strtoupper($language->language);
    $data['country']          = $country;
    $data['country_code']     = $country;
    $data['session']          = session_id();
    $data['remote_ip']        = (($ipaddr == '::1') ? '127.0.0.1' : $ipaddr);
    $data['order_no']         = $orderparam->order_id;
    $data['payment_method']   = $orderparam->payment_method;
    $data['customer_no']      = $customer_no;
    $data['orderuid']         = $orderuid;
    $data['use_utf8']         = 1;
    $data['birth_date']       = '';
    $data['fax']              = '';
    ctype_digit(variable_get($orderparam->payment_method . '_referer_id'))?$data['referer_id'] = variable_get($orderparam->payment_method . '_referer_id'):'';
    $data['system_name']      = 'drupal-übercart';
    $data['system_version']   = VERSION . '-3.x-NN1.2.2';
    $data['system_ip']        = (($_SERVER['SERVER_ADDR'] == '::1') ? '127.0.0.1' : $_SERVER['SERVER_ADDR']);
    $data['system_url']       = url('', array('absolute' => TRUE));
    $data['input1']           = 'payment_method';
    $data['input2']           = 'orderid';
    $data['input3']           = 'orderuid';
    $data['inputval1']        = $orderparam->payment_method;
    
    $data['inputval2']        = $orderparam->order_id;
    $data['inputval3']        = $orderuid;

    if ($orderparam->payment_method == 'novalnet_prepayment' || $orderparam->payment_method == 'novalnet_invoice') {
      $data['invoice_ref']    = "BNR-" . $data['product'] . "-" . $orderparam->order_id;
    }
    (variable_get($orderparam->payment_method.'_referer_id')) ? $data['referer_id'] = variable_get($orderparam->payment_method.'_referer_id') : '';
    $refer1 = (variable_get($orderparam->payment_method.'_reference1')) ? variable_get($orderparam->payment_method.'_reference1') : '';
    $refer2 = (variable_get($orderparam->payment_method.'_reference2')) ? variable_get($orderparam->payment_method.'_reference2') : '';
    if (!empty($refer1)) {
        $data['input4'] = 'reference1';
        $data['inputval4'] = $refer1;
    }
    if (!empty($refer2)) {
        $data['input5'] = 'reference2';
        $data['inputval5'] = $refer2;
    }
    uc_validate_customer_name_mail($data);
  }
  /**
   * Set additional payment parameters
   *
   * @param  $data array
   * @param  $payment_select string
   * @return array
   */
  function uc_novalnet_add_params(&$data, $payment_select) {
    $payment_key = uc_novalnet_payment_key();
    switch ($payment_select) {
      case 'novalnet_invoice':
        $data['invoice_type']        = 'INVOICE';
        $data['key']                 = trim($payment_key[$payment_select]);
        $payment_duration            = trim(variable_get($payment_select . '_duedate', ''));
        $payment_duration            = str_replace(' ', '', $payment_duration);
        if (!preg_match("/^[0-9]+$/", $payment_duration)) {
            $payment_duration        = '';
        }
        if (isset($_SESSION['novalnet']['due_date'])) unset($_SESSION['novalnet']['due_date']);
        if (is_numeric($payment_duration)) {
          $_SESSION['novalnet']['due_date'] = date("m/d/Y", mktime(0, 0, 0, date("m"), date("d")+$payment_duration, date("Y")));
          $data['due_date']                 = date("Y-m-d", mktime(0, 0, 0, date("m"), date("d")+$payment_duration, date("Y")));
        }
        break;
      case 'novalnet_prepayment':
        $data['invoice_type']         = 'PREPAYMENT';
        $data['key']                  = trim($payment_key[$payment_select]);
        break;
        
      case 'novalnet_instantbank':
      case 'novalnet_ideal':
        $data['key']                  = trim($payment_key[$payment_select]);
        $data['user_variable_0']      = url('', array('absolute' => TRUE));
        break;

      case 'novalnet_cc':
        $data['key']                  = trim($payment_key[$payment_select]);
        $data['cc_holder']            = $_SESSION['novalnet']['cc_holder'];
        $data['cc_no']                = '';
        $data['cc_type']              = $_SESSION['novalnet']['cc_type'];
        $data['pan_hash']             = $_SESSION['novalnet']['cc_panhash'];
        $data['unique_id']            = $_SESSION['novalnet']['cc_unique'];
        $data['cc_exp_month']         = $_SESSION['novalnet']['cc_exp_month'];
        $data['cc_exp_year']          = $_SESSION['novalnet']['cc_exp_year'];
        $data['cc_cvc2']              = $_SESSION['novalnet']['cc_cvc2'];
        break;

      case 'novalnet_sepa':
        $payment_type                 = trim(variable_get($payment_select . '_paymenttype', ''));
        $data['sepa_unique_id']       = $_SESSION['novalnet']['sepa_unique'];
        $data['sepa_hash']            = $_SESSION['novalnet']['sepa_panhash'];
        $data['bank_account_holder']  = $_SESSION['novalnet']['sepa_holder'];
        $data['bank_account']         = '';
        $data['bank_code']            = '';
        $data['bic']                  = '';
        $data['iban']                 = '';
        $data['key']                  = $payment_key[$payment_select];
        $data['sepa_due_date']        = uc_novalnet_sepa_duedate();
        $data['iban_bic_confirmed']     = $_SESSION['novalnet']['ibanbic_confirmed'];
        break;
      case 'novalnet_paypal':
        $data['user_variable_0']      = url('', array('absolute' => TRUE));
        $api_signature                = trim(variable_get($payment_select . '_apisignature', ''));
        $api_user                     = trim(variable_get($payment_select . '_apiuser', ''));
        $api_pw                       = trim(variable_get($payment_select . '_apipassword', ''));
        $accesskey                    = trim(variable_get($payment_select . '_accesskey', ''));
        $data['key']                  = trim($payment_key[$payment_select]);
        $data['api_signature']        = uc_novalnet_encode_value($api_signature, $accesskey);
        $data['api_user']             = uc_novalnet_encode_value($api_user, $accesskey);
        $data['api_pw']               = uc_novalnet_encode_value($api_pw, $accesskey);
        break;

      case 'novalnet_telephone':
        $data['key']                 = trim($payment_key[$payment_select]);
        break;
    }
  }
  /**
   * Set sepa due date
   *
   * @param none
   * @return date
   */
  function uc_novalnet_sepa_duedate() {
    $duedate = trim(variable_get('novalnet_sepa_duedate', '7'));
    return date("Y-m-d", mktime(0, 0, 0, date("m"), date("d")+$duedate, date("Y")));
  }

  /**
   * Set the transaction comments to order
   *
   * @param  $data array
   * @param  $order array
   * @param  $key string
   * @return string
   */
  function uc_novalnet_getcomments($data, $order, $key = 0, $requestdata='') {
    $_response = $data;
    $newline ='<br>';
    $comment = '';
    $payment_types_enc = array(
                          'novalnet_paypal',
                          'novalnet_instantbank',
                          'novalnet_ideal'
                          );
    if (isset($data['payment_method'])) {
      if (in_array($data['payment_method'], $payment_types_enc)) {
        uc_novalnet_decode($_response, $key);
      }
    }
    $space    = ' ';
    $test_mode = variable_get($order['payment_method'] . '_testmode', 0);
    if ($_response['test_mode'] == 1 || $test_mode == 1) {
      $comment =  $space . t('TESTORDER') . $newline;
    }
    if ($order['payment_method'] == 'novalnet_prepayment' || $order['payment_method'] == 'novalnet_invoice') {
      $comment .= uc_novalnet_invoice_comments($data, $order['payment_method']);
    }
    else {
       ($order['payment_method'] == 'novalnet_paypal') ? $comment .= 'Novalnet PayPal'.$newline:'';
       ($order['payment_method'] == 'novalnet_instantbank') ? $comment .= t('Novalnet Instant Bank Transfer').$newline:'';
       ($order['payment_method'] == 'novalnet_ideal') ? $comment .= t('Novalnet iDEAL').$newline:'';
       ($order['payment_method'] == 'novalnet_sepa') ? $comment .= t('Novalnet Direct Debit SEPA').$newline:'';
       ($order['payment_method'] == 'novalnet_cc') ? $comment .= t('Novalnet Credit Card').$newline:'';
       ($order['payment_method'] == 'novalnet_telephone') ? $comment .= t('Novalnet Telephone Payment').$newline:'';
            
      $comment .= t('Novalnet Transaction ID :') . ' ' . $data['tid'] . $newline;
    }

    if (isset($comment))
    drupal_set_message($comment);
    $comment= str_replace("<br>", "\n", $comment);
  return $comment;
  }

  /**
   * Set the additional transaction comments for Prepayment & Invoice
   *
   * @param  $data array
   * @param  $order array
   * @return string
   */
  function uc_novalnet_invoice_comments($data, $order) { 

    $blank_space = ' ';
    $newline ='<br>';
    $nn_prepayment_comments = '';
    $comments_end = '';
    $comments = '';
    if ($order == 'novalnet_prepayment') {
      $comments = t('Novalnet Prepayment') . $newline;
      $nn_prepayment_comments = '<div id=nn_prepayment_comments>';
      $comments_end = '</div>';
    }
    elseif ($order == 'novalnet_invoice') {
      $comments = t('Novalnet Invoice') . $newline;
    }
    $comments .= t('Novalnet Transaction ID :') . $blank_space . $data['tid'] . $newline . $newline;
    $comments .= $nn_prepayment_comments.t('Please transfer the amount to the following information to our payment service Novalnet AG') . $newline;
    if ($order == 'novalnet_invoice')
    $comments .= t('Due date :') . $blank_space . date('m/d/Y', strtotime($data['due_date'])) . $newline;
    $comments .= t('Account holder :') . $blank_space . 'NOVALNET AG' . $newline;
    $comments .= t('IBAN :') . $blank_space . $data['invoice_iban'] . $newline;
    $comments .= t('BIC :') . $blank_space . $data['invoice_bic'] . $newline;
    $comments .= t('Bank :') . $blank_space . $data['invoice_bankname'] . $blank_space . trim($data['invoice_bankplace']) . $newline;
    $comments .= t('Amount :') . $blank_space . uc_currency_format($data['amount']) . $newline;
    $comments .= t('Reference :') . $blank_space . 'TID' . $blank_space . $data['tid'] . $newline . $newline. $comments_end;
   
  return $comments;
  }
  /**
   * Set the order to complete
   *
   * @param  $comment string
   * @param  $nn_method string
   * @param  $order object
   * @param  $data array
   * @return none
   */
  function uc_novalnet_writeorderdata($comment, $nn_method, $order, $data) {
    $comments = '';
    $orderDetails = uc_order_load($order->order_id);
    $order_balance = uc_payment_balance($orderDetails);
    if ($nn_method == 'novalnet_prepayment' || $nn_method == 'novalnet_invoice') {
      $order->order_total = 0;
    }
    $payment_amount   = $order->order_total;
    $payment_currency = $order->currency;
    $context          = array(
                        'revision' => 'formatted-original',
                        'type' => 'amount',
                      );
    $options          = array('sign' => FALSE);
    if ($order->uid == '') {
      $order->uid = 0;
    }
    $query = db_update('uc_order_comments');
    $query->expression('message', "CONCAT(message, '\n" . $comment . "')");
    $query->expression('message', "CONCAT(message, '\n" . $comment . "')");
    $query->condition('order_id', $order->order_id, '=');

    if($nn_method == 'novalnet_prepayment' || $nn_method == 'novalnet_invoice')
    $order_status = variable_get($nn_method.'_before_order_status');
    else
    $order_status = variable_get($nn_method. '_order_status');

    uc_order_comment_save($order->order_id,  0,  $comment ,  'order', $order_status , 'TRUE');
    uc_payment_enter($order->order_id,  $nn_method,  $order->order_total,  $order->uid,  NULL,  $comment);
    uc_order_update_status($order->order_id, $order_status);
    if (isset($_SESSION['novalnet'])) {
      unset($_SESSION['novalnet']);
    }
  }

  /**
   * Get the payment form
   *
   * @param  $order object
   * @return array
   */
  function uc_novalnet_userform($order) {
    $form = array();

    if ($order->payment_method == 'novalnet_telephone') {
      $form = uc_novalnet_build_telephone_form($order);
    }
    elseif ($order->payment_method == 'novalnet_cc') {
      $form = uc_novalnet_build_cc_form($order);
    }
    elseif ($order->payment_method == 'novalnet_sepa') {
      $form = uc_novalnet_build_sepa_form($order);
    }
    else {
      $form[$order->payment_method . '_info'] = array('#markup' => uc_novalnet_payment_desc($order->payment_method));
    }
    if (variable_get($order->payment_method . '_testmode', '')) {
      $form[$order->payment_method . '_testmode'] = array('#markup' => uc_novalnet_payment_testmode($order->payment_method));
    }
  return $form;
  }
  /**
   * Validate the param on iframe
   *
   * @param  $iframe_params array
   * @param  $ordertotal double
   * @param  $payment_select string
   * @return none
   */
  function uc_iframe_config_validation(&$iframe_params, $ordertotal, $payment_select) {
    $vendor_id        = trim(variable_get($payment_select . '_vendorid', ''));
    $auth_code        = trim(variable_get($payment_select . '_authcode', ''));
    $productid        = trim(variable_get($payment_select . '_productid', ''));
    $tariffid         = trim(variable_get($payment_select . '_tariffid', ''));
    $manuallimit      = intval(trim(variable_get($payment_select . '_manualamount', '')));
    $productid_2      = trim(variable_get($payment_select . '_secondproduct', ''));
    $tariffid_2       = trim(variable_get($payment_select . '_secondtariff', ''));
    
    if (empty($vendor_id) || empty($auth_code) || empty($productid) || empty($tariffid)) {
     unset($_SESSION['nn']);
     return t('Basic parameter not valid');
    }
    elseif ($manuallimit) {
      if (empty($productid_2) || empty($tariffid_2)) {
      unset($_SESSION['nn']);
     return t('Basic parameter not valid');
      }
      $productid = (($ordertotal >= $manuallimit ) && isset($manuallimit) && is_numeric($manuallimit) && ($manuallimit > 0)) ?
                        $productid_2 : $productid;
    }
    
    if($payment_select == 'novalnet_sepa') {
    $iframe_params['vendor_id'] = $vendor_id;
    $iframe_params['product_id'] = $productid;
    $iframe_params['authcode'] =  $auth_code ;
    } 
    else {
    $iframe_params['nn_vendor_id_nn'] = $vendor_id;
    $iframe_params['nn_product_id_nn'] = $productid;
    $iframe_params['nn_authcode_nn'] =  $auth_code ;
    }
    return '';

  }
   
  /**
   * Get the Credit Card form from server
   *
   * @param  $order object
   * @return array
   */
  function uc_novalnet_build_cc_form($order) {
    global $base_url, $language;
    $refill_status = variable_get('novalnet_cc_refill');
    $iframe_params['nn_hash']  = (isset($_SESSION['nn']['nn_hash']) && $refill_status == 1) ? $_SESSION['nn']['nn_hash'] : '';
    $iframe_params['fldVdr']   = (isset($_SESSION['nn']['uc_novalnet_fldVdr']) && !empty($iframe_params['nn_hash'])) ? $_SESSION['novalnet']['uc_novalnet_fldVdr'] :''; 
    $ordertotal     = uc_novalnet_orderamt2novalnetamt($order); 
    $get_path = $GLOBALS['base_url'] . '/' . drupal_get_path('module', 'uc_novalnet');
    $path         = $base_url . '/novalnetcciframe?ordertotal='.$ordertotal;
    $frm = '<div id="loading_image"><img src="' . $get_path . '/img/novalnet-loading-icon.gif" /></div><iframe src="' . $path . '"  height="240" width="100%" id="nn_iframe" name="nn_iframe" style="border:none;" onload="doHideLoadingImageAndDisplayIframe(this);" frameborder="0" scrolling="no">';
    
    $frm .= '<p>Your browser does not support iframes.</p></iframe>';
    $frm .= '<script type="text/javascript" language="javascript">
        function doHideLoadingImageAndDisplayIframe(element) {
          document.getElementById("loading_image").style.display="none";
          element.style.display="";
          var iframe = (element.contentWindow || element.contentDocument);
          if (iframe.document) iframe=iframe.document;
          var getInputForm = document.getElementById("original_vendor_id");
          var formid = getInputForm.form.getAttribute("id");
          document.getElementById(formid).onsubmit = function() {
          document.getElementById("uc_novalnet_cardtype_id").value = iframe.getElementById("novalnetCc_cc_type").value;
          document.getElementById("uc_novalnet_panhash_id").value = iframe.getElementById("nncc_cardno_id").value;
          document.getElementById("uc_novalnet_unique_id").value = iframe.getElementById("nncc_unique_id").value;
          document.getElementById("uc_novalnet_card_cvv_id").value = iframe.getElementById("novalnetCc_cc_cid").value;            
          document.getElementById("uc_novalnet_cardholder_id").value = iframe.getElementById("novalnetCc_cc_owner").value;
          document.getElementById("uc_novalnet_card_expmonth_id").value = iframe.getElementById("novalnetCc_expiration").value;
          document.getElementById("uc_novalnet_card_expyear_id").value = iframe.getElementById("novalnetCc_expiration_yr").value;
          var cc_type=0; var cc_owner=0; var cc_no=0; var cc_hash=0; var cc_month=0; var cc_year=0; var cc_cid=0;
          if(iframe.getElementById("novalnetCc_cc_type").value!= "") cc_type = 1;
          if(iframe.getElementById("novalnetCc_cc_owner").value!= "") cc_owner = 1;
          if(iframe.getElementById("novalnetCc_cc_number").value!= "") cc_no = 1;
          if(iframe.getElementById("novalnetCc_expiration").value!= "") cc_month = 1;
          if(iframe.getElementById("novalnetCc_expiration_yr").value!= "") cc_year = 1;
          if(iframe.getElementById("novalnetCc_cc_cid").value!= "") cc_cid = 1;

            document.getElementById("uc_novalnet_fldVdr").value      = cc_type+","+cc_owner+","+cc_no+","+cc_month+","+cc_year+","+cc_cid;

          }
       }
    </script>';
    $form = array();

    $form['uc_novalnet_cciframe'] = array('#markup' => $frm);
    $form['cc_payment_desc']      = array(
      '#markup'    =>      uc_novalnet_payment_desc($order->payment_method),
    );
    $form['uc_novalnet_cardtype'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_cardtype_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_cardholder'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_cardholder_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_panhash'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_panhash_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_unique'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_unique_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_fldVdr'] = array(
            '#type'         => 'hidden',
            '#attributes'   => array('id' => 'uc_novalnet_fldVdr'),
    );
    $form['uc_novalnet_card_expmonth'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_card_expmonth_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_card_expyear'] = array(
      '#type'           => 'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_card_expyear_id'),
      '#default_value'  => '',
    );
    $form['uc_novalnet_card_cvv'] = array(
      '#type'           =>  'hidden',
      '#attributes'     => array('id' => 'uc_novalnet_card_cvv_id'),
      '#default_value'  => '',
    );
    $form['original_vendor_id'] = array(
      '#type'           => 'hidden',
      '#value'          => trim(variable_get('novalnet_cc_vendorid', '')),
      '#attributes'     => array('id' => 'original_vendor_id'),
      '#default_value'  => '',
    );
    $form['original_vendor_authcode'] = array(
      '#type'           => 'hidden',
      '#value'          => trim(variable_get('novalnet_cc_authcode', '')),
      '#attributes'     => array('id' => 'original_vendor_authcode'),
      '#default_value' => '',
    );
    if (defined('NOVALNET_CC_CUSTOM_CSS') && NOVALNET_CC_CUSTOM_CSS != '') {
        $form['original_cc_customstyle_css'] = array(
            '#type' => 'hidden',
            '#value' => NOVALNET_CC_CUSTOM_CSS,
            '#attributes' => array('id' => 'original_cc_customstyle_css',),
        );
        $form['original_cc_customstyle_cssval'] = array(
            '#type' => 'hidden',
            '#value' => NOVALNET_CC_CUSTOM_CSS_STYLE,
            '#attributes' => array(
                'id' => 'original_cc_customstyle_cssval',
            ),
        );
    }

  return $form;
  }
  
function uc_novalnet_form_alter(&$form, &$form_state, $form_id) {  
    drupal_add_js(drupal_get_path('module', 'uc_novalnet') . '/sepa.js', 'file');   
}

   function uc_novalnet_build_sepa_form($order) {
    global $base_url;

    $get_path = $GLOBALS['base_url'] . '/' . drupal_get_path('module', 'uc_novalnet');
    $refill_status = variable_get('novalnet_sepa_refill');
    
    $iframe_params['panhash'] = (isset($_SESSION['nn']['panhash']) && $refill_status == 1) ? $_SESSION['nn']['panhash'] : '';
    $iframe_params['fldVdr'] = (isset($_SESSION['nn']['uc_novalnet_sepa_fldVdr']) && !empty($iframe_params['panhash'])) ? $_SESSION['novalnet']['uc_novalnet_sepa_fldVdr'] : '';

    $ordertotal = uc_novalnet_orderamt2novalnetamt($order);
    $email_address = $order->primary_email;

    $country  = $order->billing_country;
    $name             =  $order->billing_first_name . ' ' .  $order->billing_last_name ;
    $city              =  $order->billing_city;
    $zip              =  $order->billing_postal_code;
    $company          =  isset($order->billing_company) ? $order->billing_company : '';
    $address =  $order->billing_street1;
    if(isset($order->billing_street2) && !empty($order->billing_street2))
    $address .= $order->billing_street2;

    $path_src = $base_url . '/novalnetsepaiframe?ordertotal=' . $ordertotal;
    $path           = $base_url . '/novalnetsepaiframe?ordertotal=' . $ordertotal .'&nn_country='.$country .'&nn_name=' .$name . '&nn_address=' .$address . '&nn_zip=' .$zip . '&nn_city=' . $city .'&nn_email=' . $email_address . '&nn_company=' . $company;  
    
    $frm = '<div id="loading_image"><img src="' . $get_path . '/img/novalnet-loading-icon.gif" /></div><iframe src="' . $path . '"  scrolling ="no" height="430" width="100%" id="nn_iframe" name="nn_iframe" style="border:none;" onload="doHideLoadingImageAndDisplayIframe(this);" frameborder="0">';
    $frm .= '<p>Your browser does not support iframes.</p></iframe>';

    $form = array();
    $form['uc_novalnet_sepa_src'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_src'),
        '#default_value' => $path_src,
    );

    $form['uc_novalnet_sepaiframe'] = array('#markup' => $frm);
    $form['sepa_payment_desc'] = array(
        '#markup' => t('Your account will be debited upon delivery of goods.'),
    );
    $form['uc_novalnet_sepaowner'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_owner_id'),
        '#default_value' => '',
    );

    $form['uc_novalnet_sepa_panhash'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_panhash_id'),
        '#default_value' => '',
    );
    $form['uc_novalnet_sepa_unique'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_unique_id'),
        '#default_value' => '',
    );

    $form['uc_novalnet_sepa_confirm'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_confirm_id'),
        '#default_value' => '',
    );
    $form['uc_novalnet_sepa_fldVdr'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_fldVdr'),
    );
    $form['original_vendor_id'] = array(
        '#type' => 'hidden',
        '#value' => trim(variable_get('novalnet_sepa_vendorid', '')),
        '#attributes' => array('id' => 'original_vendor_id'),
        '#default_value' => '',
    );
    $form['uc_novalnet_sepa_mandate_ref'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_mandate_ref_id'),
        '#default_value' => '',
    );
    $form['uc_novalnet_sepa_mandate_date'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_mandate_date_id'),
        '#default_value' => '',
    );
    $form['uc_novalnet_sepa_user_email'] = array(
        '#type' => 'hidden',
        '#attributes' => array('id' => 'uc_novalnet_sepa_user_email'),
        '#default_value' => $email_address,
    );
    if (defined('NOVALNET_SEPA_CUSTOM_CSS') && NOVALNET_SEPA_CUSTOM_CSS != '') {
        $form['original_sepa_customstyle_css'] = array(
            '#type' => 'hidden',
            '#value' => NOVALNET_SEPA_CUSTOM_CSS,
            '#attributes' => array('id' => 'original_sepa_customstyle_css',),
        );
        $form['original_sepa_customstyle_cssval'] = array(
            '#type' => 'hidden',
            '#value' => NOVALNET_SEPA_CUSTOM_CSS_STYLE,
            '#attributes' => array(
                'id' => 'original_sepa_customstyle_cssval',
            ),
        );
    }
    return $form;
}

  /**
   * Set the telephone payment form
   *
   * @param  $order object
   * @return array
   */
  function uc_novalnet_build_telephone_form($order) { 
    $form = array();

    if (isset($_SESSION['novalnet'][$order->payment_method]['tid'])) {
      
      if (($_SESSION['novalnet'][$order->payment_method]['order_id'] != $order->order_id) && ($_SESSION['novalnet'][$order->payment_method]['order_total'] == $order->order_total)) {
        unset($_SESSION['novalnet']);
        $form[$order->payment_method . '_info'] = array('#markup' => uc_novalnet_payment_desc($order->payment_method));
      }
      
      elseif ($_SESSION['novalnet'][$order->payment_method]['order_total'] != $order->order_total) {
       
        if(variable_get($order->payment_method.'_fraud_prevention', '') == 'callback') 
        drupal_set_message(t('You have changed the order amount after getting PIN number, please try again with a new call'), 'warning');
        if(variable_get($order->payment_method.'_fraud_prevention', '') == 'email') 
        drupal_set_message(t('You have changed the order amount after getting e-mail, please try again with a new call'), 'warning');
        if($order->payment_method == 'novalnet_telephone')
        drupal_set_message(t('You have changed the order amount after receiving telephone number, please try again with a new call'), 'warning');
        unset($_SESSION['novalnet']);
        $form = fraud_prevention_form($order);
        $form[$order->payment_method . '_info'] = array('#markup' => uc_novalnet_payment_desc($order->payment_method));
      }
      
      else {
      if($order->payment_method == 'novalnet_telephone') {
        $sess_tel = trim($_SESSION['novalnet'][$order->payment_method]['tel_no']);
        if ($sess_tel) {
          $tel_digits = str_split($sess_tel, 4);
          $count = 0;
          $str_sess_tel = '';
          foreach ($tel_digits as $ind => $digits) {
            $count++;
            $str_sess_tel .= $digits;
            if ($count==1) $str_sess_tel .= '-';
            else $str_sess_tel .= ' ';
          }
          $str_sess_tel=trim($str_sess_tel);
          if ($str_sess_tel) $sess_tel=$str_sess_tel;
        }
        $desc = '<div>' . t('Your amount will be added in your telephone bill when you place the order.') . '</div><br />';
        $desc .= '<div>' . t('Following steps are required to complete your payment:') . '</div><br />';
        $desc .= '<div>' . t('Step 1:') . ' ' . t('Please call the telephone number displayed:') . ' <b>' . $sess_tel . '</b>.<br />' . t('* This call will cost') . ' <b>' . uc_currency_format($_SESSION['novalnet'][$order->payment_method]['order_total']) . ' </b> ' . t('(including VAT) and it is possible only for German landline connection! *') . '</div><br />';
        $desc .= '<div>' . t('Step 2:') . ' ' . t('Please wait for the beep and then hang up the listeners.') . '<br>' . t('After your successful call, please proceed with the payment.');

        $form[$order->payment_method . '_info'] = array('#markup' => $desc);
       } 
       
       else if(variable_get($order->payment_method.'_fraud_prevention', '') != 'false') {
        if(variable_get($order->payment_method.'_fraud_prevention', '') == 'email') 
            if(!isset($_SESSION['novalnet'][$order->payment_method]['nn_error']))   
            drupal_set_message(t('We have sent a email, please answer.'));
         
        if(variable_get($order->payment_method.'_fraud_prevention', '') != 'email') {
          if(!isset($_SESSION['novalnet'][$order->payment_method]['nn_error'])) 
          drupal_set_message(t('You will shortly receive a PIN by phone / SMS. Please enter the PIN in the appropriate text box.'));
          $form[$order->payment_method . '_pin'] = array(
          '#type'             => 'textfield',
          '#title'            => t('PIN Number'),
          '#element_validate' => array('uc_novalnet_fraudprevention_data_validation'),
          '#attributes'       => array('autocomplete' => 'off')
          );
          $form[$order->payment_method . '_forget_pin'] = array(
          '#type'             => 'checkbox',
          '#title'            => t('Forget PIN'),
          );
         }
       }       
      }
    }

    else if($order->payment_method != 'novalnet_telephone' && (variable_get($order->payment_method.'_fraud_prevention', '') != 'false') || isset($_SESSION['novalnet']['rebuild'])){
        $form = fraud_prevention_form($order);
    }
    
    else{
    $form[$order->payment_method . '_info'] = array('#markup' => uc_novalnet_payment_desc($order->payment_method)); 
    }
  return $form;
  }
  
  function fraud_prevention_form($order){ 
	  $form = array();
      if($order->payment_method != 'novalnet_telephone' && (variable_get($order->payment_method.'_fraud_prevention', '') != 'email'))
      $form[$order->payment_method . '_tel'] = array(
      '#type'             => 'textfield',
      '#title'            => t('Phone Number'),
      '#default_value'    => isset($order->billing_phone) ? $order->billing_phone : '',
      '#attributes'       => array('autocomplete' => 'off')
      );
      if(variable_get($order->payment_method.'_fraud_prevention', '') == 'email')
      $form[$order->payment_method . '_email'] = array(
       '#type'             => 'textfield',
       '#title'            => t('E-mail'),
       '#default_value'    => isset($order->primary_email) ? $order->primary_email : '',
       '#attributes'       => array('autocomplete' => 'off')
      );
    return $form;  
  }

  /**
   * Set the Direct Debit form
   *
   * @param  $novalnet_payment String
   * @return array
   */
  function uc_novalnet_build_form($novalnet_payment) {
    global $is_https;

    $form = array();

    $form['account_holder'] = array(
      '#type'       => 'textfield',
      '#title'      => t('Account holder'),
      '#required'   => TRUE,
      '#value'      => variable_get('account_holder',  ''),
      '#attributes' => array('autocomplete' => 'off'),
    );
    $form['account_no'] = array(
      '#type'       => 'textfield',
      '#title'      => t('Account number'),
      '#required'   => TRUE,
      '#value'      => variable_get('account_holder',  ''),
      '#attributes' => array('autocomplete' => 'off'),
    );
    $form['bank_code'] = array(
      '#type'       => 'textfield',
      '#title'      => t('Bankcode'),
      '#required'   => TRUE,
      '#value'      => variable_get('account_holder',  ''),
      '#attributes' => array('autocomplete' => 'off'),
    );
    if ($novalnet_payment == 'novalnet_elv_de' && variable_get($novalnet_payment . '_acdc',  '')) {
      $form['acdc_check'] = array(
        '#type'       =>  'checkbox',
        '#checked'    =>  FALSE,
        '#title'      =>  filter_xss('<a href="' . $acdc_url . '" target="_blank" >' . check_plain(t('Your credit rating is checked by us')) . '</a>'),
        '#attributes' => array(
        'onmouseover' => 'sample_hover()',
      ),
    );
    }
    $form[$novalnet_payment . '_info'] = array(
      '#markup' => uc_novalnet_payment_desc($novalnet_payment)
    );


  return $form;
  }
  /**
   * Check the openssl status
   *
   * @param  none
   * @return none
   */
  function check_php_extension() {
    $php_extn_error = '';
    if (!extension_loaded('openssl')) {
      $php_extn_error = 'Please install/enable Open SSL PHP extension!';
    }

    if ($php_extn_error != '') {
      drupal_set_message(check_plain(t($php_extn_error)), 'error');
      drupal_goto('cart/checkout');
      exit;
    }
  }
  
   /**
   * Set ipaddress
   *
   * @param  $value string
   * @return string
   */
   
  function public_ip($value) {
    return (count(explode(' . ',  $value)) == 4 && !preg_match('~^((0|10|172\ . 16|192\ . 168|169\ . 254|255|127\ . 0)\ . )~',  $value));
  }

  /** Function for Getting the Ip Address **/
  function get_realip() {
    $_check = array(
      'HTTP_X_FORWARDED_FOR',
      'HTTP_CLIENT_IP',
      'HTTP_X_CLUSTER_CLIENT_IP',
      'HTTP_FORWARDED_FOR',
      );
    foreach ($_check as $_key) {
      if (isset($_SERVER[$_key])) {
        $ips = explode(', ',  $_SERVER[$_key]);
        if (isset($ips[0]) && public_ip($ips[0])) {
          return $ips[0];
        }
      }
    }
  return $_SERVER['REMOTE_ADDR'];
  }

  function uc_novalnet_error_redirect() {
    drupal_goto('cart/checkout');
    exit;
  }

  /**
   * Check the response status
   *
   * @param  $data array
   * @return boolean/string
   */
  function uc_novalnet_check_status($data) {
    if (isset($data['status']) && $data['status'] == '100') {
      return TRUE;
    }
    elseif (isset($data['status']) && $data['status']=='90' && $data['payment_method'] == 'novalnet_paypal') {
      return TRUE;
    }
    else {
      $novalneterror = uc_get_novalnet_payment_error($data);
    return $novalneterror;
    }
  }

  /**
   * Get the payment error from response
   *
   * @param  $data array
   * @return string
   */
  function uc_get_novalnet_payment_error($data) {
    if (isset($data['status_text'])) {
      $uc_novalnet_error = $data['status_text'];
    }
    elseif (isset($data['status_desc'])) {
      $uc_novalnet_error = $data['status_desc'];
    }
    elseif (isset($data['novaltel_status_message'])) {
      $uc_novalnet_error = $data['novaltel_status_message'];
    }
    else{
      $uc_novalnet_error = t('There was an error and your payment could not be completed.') . '(' . $data['status'] . ')';
    }
  return $uc_novalnet_error;
  }

  /**
   * Validate the user data
   *
   * @param  $data array
   * @param  $userdata array
   * @param  $payment_select string
   * @return array / string
   */
  function uc_novalnet_sanitizebankdata(&$data, $userdata, $payment_select) { 
     $userdata = array_map('trim', $userdata);
     if ($payment_select == 'novalnet_sepa'){
        $refill_status = variable_get($payment_select.'_refill');
        if(isset($refill_status) && $refill_status == 1 && !empty($_REQUEST['panes']['payment']['details']['uc_novalnet_sepa_panhash']) ){
        $_SESSION['nn']['panhash']  = (isset( $_REQUEST['panes']['payment']['details']['uc_novalnet_sepa_panhash'] )) ? $_REQUEST['panes']['payment']['details']['uc_novalnet_sepa_panhash'] : '';
            $_SESSION['nn']['sepa_fldVdr']   = isset( $refill_status )? $_REQUEST['panes']['payment']['details']['uc_novalnet_sepa_fldVdr']:'';
      }
      if (isset($userdata['uc_novalnet_sepaowner']) && isset($userdata['uc_novalnet_sepa_panhash']) && isset($userdata['uc_novalnet_sepa_unique']) && isset($userdata['uc_novalnet_sepa_confirm'])) {
         $sepaholder = trim($userdata['uc_novalnet_sepaowner']);

        if ($userdata['uc_novalnet_sepa_confirm'] != 1) {
          return t('Please confirm IBAN & BIC');
        }
        if (empty($sepaholder) || preg_match('/[#%\^<>@=*!]/',  $sepaholder) ||
            empty($userdata['uc_novalnet_sepa_panhash']) || empty($userdata['uc_novalnet_sepa_unique'])) {
          return t('Please enter valid account details!');
        }
        $_SESSION['novalnet']['sepa_holder']          = $userdata['uc_novalnet_sepaowner'];
        $_SESSION['novalnet']['sepa_panhash']         = $userdata['uc_novalnet_sepa_panhash'];
        $_SESSION['novalnet']['sepa_unique']          = $userdata['uc_novalnet_sepa_unique'];
        $_SESSION['novalnet']['ibanbic_confirmed']    = $userdata['uc_novalnet_sepa_confirm'];

      }
    }
    if(isset($_SESSION['novalnet'][$payment_select]['fraud_prevention'])) {
        if(isset($userdata[$payment_select.'_tel']) && !is_numeric($userdata[$payment_select.'_tel'])) {
            return t('Please enter the Telephone / Mobilenumber');
        }
        if(isset($userdata[$payment_select.'_email']) && !valid_email_address($userdata[$payment_select.'_email'])) {
            return t('Please enter the E-Mail Address');
        }   
    }   
    if ($payment_select == 'novalnet_cc') {
        $refill_status = variable_get($payment_select.'_refill');
        if(isset($refill_status) && $refill_status == 1 && !empty($_REQUEST['panes']['payment']['details']['uc_novalnet_panhash'])){
            $_SESSION['nn']['nn_hash']  = (isset( $_REQUEST['panes']['payment']['details']['uc_novalnet_panhash'] )) ? $_REQUEST['panes']['payment']['details']['uc_novalnet_panhash'] : '';
            $_SESSION['nn']['fldVdr']   = isset( $refill_status )? $_REQUEST['panes']['payment']['details']['uc_novalnet_fldVdr']:''; 
        }
      if (isset($userdata['uc_novalnet_cardholder'])  && isset($userdata['uc_novalnet_card_cvv']) && isset($userdata['uc_novalnet_card_expyear']) && isset($userdata['uc_novalnet_card_expmonth']) && isset($userdata['uc_novalnet_panhash']) && isset($userdata['uc_novalnet_unique'])) {
        $cc_type      = $userdata['uc_novalnet_cardtype'];
        $cc_holder    = $userdata['uc_novalnet_cardholder'];
        $cc_cvc       = $userdata['uc_novalnet_card_cvv'];
        $cc_exp_year  = $userdata['uc_novalnet_card_expyear'];
        $cc_exp_month = $userdata['uc_novalnet_card_expmonth'];

        if (empty($cc_holder) || preg_match('/[#%\^<>@=*!]/',  $cc_holder)) {
          return t('Please enter valid credit card details!');
        }
        elseif (empty($userdata['uc_novalnet_panhash']) || empty($userdata['uc_novalnet_unique'])) {
          return t('Please enter valid credit card details!');
        }
          elseif (empty($cc_cvc)) {
          return t('Please enter valid credit card details!');
        }
        elseif (empty($cc_exp_year) || empty($cc_exp_month) || ($cc_exp_year < date('Y')) || ($cc_exp_year == date('Y') && $cc_exp_month < date('m'))) {
          return t('Please enter valid credit card details!');
        }
        else {
          $_SESSION['novalnet']['cc_holder']    = $cc_holder;
          $_SESSION['novalnet']['cc_panhash']   = $userdata['uc_novalnet_panhash'];
          $_SESSION['novalnet']['cc_unique']    = $userdata['uc_novalnet_unique'];
          $_SESSION['novalnet']['cc_exp_month'] = $cc_exp_month;
          $_SESSION['novalnet']['cc_exp_year']  = $cc_exp_year;
          $_SESSION['novalnet']['cc_cvc2']      = $cc_cvc;
          $_SESSION['novalnet']['cc_type']      = $cc_type;
          
        return '';
        }
      }
      else {
        return t('Please enter valid credit card details!');
      }
    } 
  }
  /** Method to validate numeric values **/
  function uc_novalnet_sanitizeNumber($number) {
    return preg_replace('/[\s]+/',  '',  $number);
  }
  /** Method to validate the credit card expiry period **/
  function uc_novalnet_validateCcExpiry($month,  $year) {
    $tmp = strtotime('-3 days');
    return ($month > 0 && $month < 13 && ($year == date('Y',  $tmp) ? ($month >= date('n',  $tmp)) : ($year > date('Y',  $tmp))));
  }

  /**
   * Get the payment access key
   *
   * @param  $order_payment string
   * @return string
   */
  function uc_novalnet_access_key($order_payment) {
    $key = trim(variable_get($order_payment . '_accesskey', ''));
    if ($key == '' && empty($key)) {
      $nnencodeerror = check_plain(t('Basic parameter not valid'));
      drupal_set_message(check_plain(t('Basic parameter not valid')), 'warning');
      watchdog('uc_novalnet', $order_payment, array('@error' => $nnencodeerror), WATCHDOG_ERROR);
      drupal_goto('cart/checkout');
      exit;
    }
    return $key;
  }
   /**
   * Set the data to encode
   *
   * @param  $order_payment string
   * @param  $data array
   * @param  $key string
   * @return string/array
   */
  function uc_novalnet_encode_data(&$data, $key, $order_payment) {
    $afterencode = uc_novalnet_encode($data, $key);
    if ($afterencode != 1) {
      $nnencodeerror = check_plain(t('Encoding Error, Payment System Not avillable now, Choose Another'));
      drupal_set_message(check_plain(t('Encoding Error, Payment System Not avillable now, Choose Another')), 'warning');
      watchdog('uc_novalnet', $order_payment , array('@error' => $nnencodeerror), WATCHDOG_ERROR);
      drupal_goto('cart/checkout');
      exit;
    }
  }
   /**
   * Encode the Parameters
   *
   * @param  $data array
   * @param  $key string
   * @return string/array
   */
  function uc_novalnet_encode(&$data,  $key) {
    if (!function_exists('base64_encode') || !function_exists('pack') || !function_exists('crc32')) {
    return FALSE;
    }
    $encoded = array('vendor_authcode',  'product_id',  'tariff_id',  'test_mode',  'uniqid',  'amount');
    foreach ($encoded as $_value ) {
      $fields = $data[$_value];
      if ($fields == '') {
        return FALSE;
      }
      try {
        $crc = sprintf('%u',  crc32($fields));//%u is a must for ccrc32 returns a signed value
        $fields = $crc . "|" . $fields;
        $fields = bin2hex($fields . $key);
        $fields = strrev(base64_encode($fields));
        $data[$_value] = $fields;
      } catch (Exception $e) {
        return FALSE;
      }
    }
  return TRUE;
  }
  /**
   * Set return url's for redirection methods
   *
   * @param  $data array
   * @return string/array
   */
  function uc_novalnet_geturl(&$data) {
    $return_url                   = url('nnpayment/review',  array('absolute' => TRUE));
    $return_method                = 'POST';
    $data['return_url']           = $return_url;
    $data['return_method']        = $return_method;
    $data['error_return_url']     = $return_url;
    $data['error_return_method']  = $return_method;
  }
  /**
   * Set the hash value
   *
   * @param  $data array
   * @param  $key string
   * @return string
   */
  function uc_novalnet_hashIt($data, $key) {
    $hashp = array(
      'product'    => $data['product_id'],
      'tariff'     => $data['tariff_id'],
      'auth_code'  => $data['vendor_authcode'],
      'amount'     => $data['amount'],
      'test_mode'  => $data['test_mode'],
      'uniqid'     => $data['uniqid'],
    );
    $hash_fields = array('auth_code',  'product',  'tariff',  'amount',  'test_mode',  'uniqid' );
    $str = NULL;
    foreach ($hash_fields as $_value) {
      if ($hashp[$_value] == '') {
        return FALSE;
      }
      $str .= $hashp[$_value];
    }
  return md5($str . strrev($key));
  }
  /**
   * Get alter the response parameters
   *
   * @param  $request array
   * @return array
   */
  function uc_novalnet_reconstruct_param_fordecode(&$request) {
      $redirection = array('novalnet_instantbank', 'novalnet_ideal', 'novalnet_paypal');
    if (in_array($request['payment_method'], $redirection)) {
        $request['product_id']      = $request['product'];
        $request['vendor_authcode'] = $request['auth_code'];
        $request['tariff_id']       = $request['tariff'];
    }
  }
  /**
   * Decode the parameter
   *
   * @param  $data array
   * @param  $key string
   * @return array/boolean
   */
 function uc_novalnet_decode_param(&$data, $key) {
    if (!function_exists('base64_decode') || !function_exists('pack') || !function_exists('crc32')) {
      return FALSE;
    }
    if ($data == '') {
      return FALSE;
    }
    try{
      $data =  base64_decode(strrev($data));
      $data = pack("H" . strlen($data),  $data);
      $data = substr($data,  0,  stripos($data,  $key));
      $pos = strpos($data,  "|");
      if ($pos === FALSE) {
        return FALSE;
      }
      $crc = substr($data,  0,  $pos);
      $value = trim(substr($data,  $pos+1));
      if ( $crc != sprintf('%u',  crc32($value)) ) {
        return FALSE;
      }

      return $value;
      } catch (Exception $e) {
        return FALSE;
      }

  return TRUE;
  }
  /**
   * Decode the parameters
   *
   * @param  $_response array
   * @param  $key string
   * @return array/boolean
   */
  function uc_novalnet_decode(&$_response,  $key) {

    if (!function_exists('base64_decode') || !function_exists('pack') || !function_exists('crc32')) {
      return FALSE;
    }
    $encoded = array('vendor_authcode',  'product_id',  'tariff_id',  'test_mode',  'uniqid',  'amount');
    foreach ($encoded as $_value ) {
      $fields = $_response[$_value];
      if ($fields == '') {
        return FALSE;
      }
      try{
        $fields =  base64_decode(strrev($fields));
        $fields = pack("H" . strlen($fields),  $fields);
        $fields = substr($fields,  0,  stripos($fields,  $key));
        $pos = strpos($fields,  "|");
        if ($pos === FALSE) {
          return FALSE;
        }
        $crc = substr($fields,  0,  $pos);
        $value = trim(substr($fields,  $pos+1));
        if ( $crc != sprintf('%u',  crc32($value)) ) {
          return FALSE;
        }

        $_response[$_value] = $value;
      } catch (Exception $e) {
        return FALSE;
      }
    }
  return TRUE;
  }
  /**
   * Get the form with hidden fields
   *
   * @param  $form_name string
   * @param  $form_elements array
   * @param  $form_action_url string
   * @return array/boolean
   */
  function drupal_nn_get_form($form_name, $form_elements, $form_action_url) {
    $form_start = '<form name="' . $form_name . '" id="' . $form_name . '" action="' . $form_action_url . '" method="post">';
    $form_elements_html = '';
    foreach ($form_elements as $key => $value) {
      $form_elements_html .= '<input type="hidden" name="' . $key . '" value="' . $value . '" id="' . $key . '" />';
    }
    $form_elements_html .= '<input type="submit" name="' . $key . '" value="' . t('You will be redirected to Novalnet AG in a few seconds.') . '" id="' . $key . '" />';
    $form_end='</form>';
    return ($form_start . $form_elements_html . $form_end);
  }
  /**
   * Get the form with hidden fields
   *
   * @param  $data array
   * @param  $form_action_url string
   * @return array/boolean
   */
  function uc_novalnet_payment_complete($data) {
    $order     = array();
    $key       = trim(variable_get($data['payment_method'] . '_accesskey', ''));
    $order     = array('order_id' => $data['order_no'], 'payment_method' => $data['payment_method'], 'orderuid' =>$data['orderuid'], 'orderuid' => $data['amount']);
    $orderparam->payment_method = $data['payment_method'];
    $orderparam->order_id       = $data['order_no'];
    $data['orderuid'] = ($data['orderuid'] == 'NULL') ? 0 : $data['orderuid'];
            
    if ($data['payment_method'] == 'novalnet_cc' || $data['payment_method'] == 'novalnet_cc3d') {
      $nn_comments  = uc_novalnet_getcomments($data, $order, $key);
      $nn_data      = uc_novalnet_writecomments($data, $nn_comments, $order, $key);
      uc_post_back_call($data, $data['tid'], $orderparam);
      $data['amount'] = uc_novalnet_get_amount($data['amount']);
      uc_payment_enter($order['order_id'],  $order['payment_method'],  $data['amount'],  $data['orderuid'],  NULL,  $nn_comments);
      return $nn_data;
    }
    $temp_data = uc_build_temp_data($data);
    
    if (uc_novalnet_checkHash($temp_data, $key) == TRUE && isset($order)) {
      $nn_comments  = uc_novalnet_getcomments($data, $order, $key);
      $nn_data      = uc_novalnet_writecomments($data, $nn_comments, $order, $key);
      uc_post_back_call($data, $data['tid'], $orderparam, $key);
      $amount = uc_novalnet_decode_param($data['amount'], $key);
      $amount = uc_novalnet_get_amount($amount);

      if($order['payment_method'] == 'novalnet_paypal' && $data['status'] == '90'){
      $amount = '0';
      }
      uc_payment_enter($order['order_id'],  $order['payment_method'],  $amount,  $data['orderuid'],  NULL,  $nn_comments);
      uc_order_update_status($order['order_id'], $nn_data['order_status']);
      return $nn_data;
    }
    return FALSE;
  }
  
  function uc_novalnet_get_amount($order_amount){
    $order_amount = round($order_amount, 2);
    return $order_amount / 100;
    }
 /**
   * Set transacation comments to database
   *
   * @param  $data array
   * @param  $comment string
   * @param  $order string
   * @param  $key string
   * @return array/boolean
   */
  function uc_novalnet_writecomments($data, $comment, $order, $key) {
    $order_data = uc_order_load($order['order_id']);
    $_response = $data;
	$order_status = variable_get($order_data->payment_method.'_order_status', '');
    if ($order_data->payment_method != 'novalnet_cc') {
      $responsedecode = uc_novalnet_decode($_response, $key);
    }
    if ($order_data->payment_method == 'novalnet_paypal' && $data['status'] == 90) {
      $order_status = variable_get('novalnet_paypal_before_order_status', '');
    }
    if($order_data->payment_method == 'novalnet_paypal' && $data['status'] == 100) {
      $order_status = variable_get('novalnet_paypal_after_order_status', '');
    }

    $payment_currency = $data['currency'];
    $context = array(
      'revision' => 'formatted-original',
      'type' => 'amount',
    );
    $options = array(
      'sign' => FALSE,
    );
    $order['payment_method'] = trim($order['payment_method']);
    if ($order['orderuid'] == 'NULL') {
      $order['orderuid'] = 0;
    }

    $query = db_update('uc_order_comments');
    $query->expression('message', "CONCAT(message, '\n" . $comment . "')");
    $query->condition('order_id', $order['order_id'], '=');
    uc_order_comment_save($order['order_id'],  0,  $comment,  'order', $order_status , 'TRUE');
    $data = array(
      'tid'           => $data['tid'],
      'status'        => $data['status'],
      'payment_type'  => $order['payment_method'],
      'created'       => time(),
    );

    //////////////////////////////////////////////////
    uc_order_update_status($order['order_id'], $order_status);

    if (isset($_SESSION['novalnet'])) {
      unset($_SESSION['novalnet']);
    }

    return $data;
  }
  /**
   * Set temporary array
   *
   * @param  $data array
   * @return array
   */
  function uc_build_temp_data($data) {
    $temp_data = array();
    $temp_data['vendor_authcode'] = $data['vendor_authcode'];
    $temp_data['product_id']      = $data['product_id'];
    $temp_data['tariff_id']       = $data['tariff_id'];
    $temp_data['test_mode']       = $data['test_mode'];
    $temp_data['uniqid']          = $data['uniqid'];
    $temp_data['amount']          = $data['amount'];
    $temp_data['hash2']           = $data['hash2'];
    $temp_data['hash']            = $data['hash'];
  return $temp_data;
  }
  /**
   * Encode the parameter value
   *
   * @param  $data array
   * @param  $key string
   * @return array
   */
   function uc_novalnet_encode_value($data,  $key) {
     if (!function_exists('base64_encode') || !function_exists('pack') || !function_exists('crc32')) {
       return FALSE;
     }
     if ($data == '') {
       return FALSE;
     }
     try {
       $crc = sprintf('%u',  crc32($data));//%u is a must for ccrc32 returns a signed value
       $data = $crc . "|" . $data;
       $data = bin2hex($data . $key);
       $data = strrev(base64_encode($data));
       return $data;
     } catch (Exception $e) {
       return FALSE;
     }
   }
   /**
   * Check the returned hash
   *
   * @param  $response array
   * @param  $key string
   * @return boolean
   */
  function uc_novalnet_checkHash($response,  $key) {
    return isset($response['hash2'])  && !(empty($response['hash2']))  && (($tmp = uc_novalnet_hashIt($response,  $key)) !== FALSE) && ($response['hash2'] == $tmp);
  }
  /**
   * Set the Novalnet transaction ID
   *
   * @param  $order_id string
   * @param  $tid double
   * @return none
   */
  function uc_novalnet_save_tid($order_id,  $tid, $uid) {
    $data = array(
    'order_id'  => $order_id,
    'tid'       => $tid,
    'uid'       => $uid,
    );
    drupal_write_record('uc_novalnet',  $data);
  }
  /**Function For entering the Novalnet Comment to the Database**/
  function uc_novalnet_novalnetamt2orderamt($nn_amt) {
    return $nn_amt/100;
  }
  /**
   * Set the sucess redirection
   *
   * @param  none
   * @return none
   */
  function uc_novalnet_success_redirect() {
    $_SESSION['uc_checkout'][$_SESSION['cart_order']]['do_complete'] = TRUE;
    $_SESSION['do_complete'] = TRUE;
    if(isset($_SESSION['nn']))  unset($_SESSION['nn']);  //unset the flDvdr session values
    drupal_goto('cart/checkout/complete');
    exit;
  }
  /**
   * Preform second call for telephone payment
   *
   * @param  $order object
   * @return none
   */

       
  function uc_perform_second_call($order) {
    $xmldata['vendor_id']         = trim(variable_get($order->payment_method . '_vendorid',  ''));
    $xmldata['vendor_authcode']   = trim(variable_get($order->payment_method . '_authcode',  ''));
    $xmldata['request_type']      = $order->request_type;
    $xmldata['nn_tid']            = $_SESSION['novalnet'][$order->payment_method]['tid'];
    if(isset($_SESSION['novalnet'][$order->payment_method]['pin']) && $order->request_type != 'TRANSMIT_PIN_AGAIN') {
    $xmldata['nn_pin']            = $_SESSION['novalnet'][$order->payment_method]['pin'];
    unset($_SESSION['novalnet'][$order->payment_method]['pin']);
    }  
    if(array_search('', $xmldata)) {
        unset($_SESSION['novalnet'][$order->payment_method]);
        drupal_set_message(t('Required parameter not valid'), 'warning');
        drupal_goto('cart/checkout');
        exit;
    }
    $options      = uc_novalnet_xmlparam($xmldata);
    $aryresponse  = uc_novalnet_second_call($options); 
    
    //Manual Testing
	#$aryresponse['status'] = 100;
	//Manual Testing

    if ($_SESSION['novalnet'][$order->payment_method]['tid'] && $aryresponse) {
      if(isset($aryresponse['status']) && $aryresponse['status'] == 100){ 
        #### On successful payment ####
        #### Redirecting the user to the checkout page ####
        $data = array();
        $data['tid']       = $_SESSION['novalnet'][$order->payment_method]['tid'];
        $data['test_mode'] = $_SESSION['novalnet'][$order->payment_method]['testmode'];
        if($order->payment_method == 'novalnet_invoice') {        
        $data['invoice_iban']      = $_SESSION['novalnet']['invoice']['invoice_iban'];
        $data['invoice_bic']       = $_SESSION['novalnet']['invoice']['invoice_bic'];
        $data['invoice_bankname']  = $_SESSION['novalnet']['invoice']['invoice_bankname'];
        $data['invoice_bankplace'] = $_SESSION['novalnet']['invoice']['invoice_bankplace'];
        $data['amount']            = $_SESSION['novalnet']['invoice']['amount'];
        $data['due_date']          = $_SESSION['novalnet']['invoice']['due_date'];
        $data['payment_method']    = $order->payment_method;
        }       
        $nn_method['payment_method'] = $order->payment_method; 
        $comments = uc_novalnet_getcomments($data, $nn_method, $key=0);
        uc_novalnet_writeorderdata($comments, $nn_method['payment_method'], $order, $data);
        $postback_data['key']           = uc_novalnet_payment_key($order->payment_method);
        $postback_data['payment_method']= $order->payment_method;
        $postback_data['vendor']        = trim(variable_get($order->payment_method . '_vendorid',  ''));
        $postback_data['product']       = trim(variable_get($order->payment_method . '_productid',  ''));
        $postback_data['tariff']        = trim(variable_get($order->payment_method . '_tariffid',  ''));
        $postback_data['auth_code']     = trim(variable_get($order->payment_method . '_authcode',  ''));
        uc_post_back_call($postback_data, $data['tid'], $order);
        if (isset($_SESSION['novalnet'][$order->payment_method])) {
          unset($_SESSION['novalnet'][$order->payment_method]);
        }
        return;
      }else{ 
          $_SESSION['novalnet'][$order->payment_method]['nn_error'] = TRUE;
            if ($order->payment_method =='novalnet_telephone' && $aryresponse['novaltel_status'] == 18) {
			  $aryresponse['status_message'] = $aryresponse['novaltel_status_message'];
              unset($_SESSION['novalnet'][$order->payment_method]['tid'],
              $_SESSION['novalnet'][$order->payment_method]['testmode'],
              $_SESSION['novalnet'][$order->payment_method]);
            }
            if(isset($aryresponse['status']) && ($aryresponse['status'] == '0529006')) {
			$_SESSION['payment_method'] = $order->payment_method;
            $_SESSION[$order->payment_method]['pin_count'] = TRUE;
			}
            if(isset($aryresponse['pin_status']['status_message']))
            $aryresponse['status_message'] = $aryresponse['pin_status']['status_message'];
            
        }
        drupal_set_message($aryresponse['status_message'], 'warning');
        drupal_goto('cart/checkout');
        exit;
      }
    }
  
  /**
   * Build xml type request
   *
   * @param  $data array
   * @return string
   */
  function uc_novalnet_xmlparam($data) {
    $xml = '<?xml version="1.0" encoding="UTF-8"?>
    <nnxml>
      <info_request>
        <vendor_id>' . $data['vendor_id'] . '</vendor_id>
        <vendor_authcode>' . $data['vendor_authcode'] . '</vendor_authcode>
        <request_type>' . $data['request_type'] . '</request_type>
        <tid>' . $data['nn_tid'] . '</tid>';
        if ($data['request_type'] == 'PIN_STATUS') {
             $xml .= '<pin>' . $data['nn_pin'] . '</pin>';
         }
        $xml .= '</info_request>
        </nnxml>';
    return $xml;
  }

  /** Method to send param to payport server second time when callback option is activate **/
  function uc_novalnet_second_call($xml) {
    $options = array(
      'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
      'method' => 'POST',
      'data' => $xml
    );
    $host = 'https://payport.novalnet.de/nn_infoport.xml';
    $response = drupal_http_request($host, $options);
    $xml = simplexml_load_string($response->data);
    return json_decode(json_encode($xml), TRUE);
  }
  /**
   * Perform post back call
   *
   * @param  $data array
   * @param  $tid double
   * @param  $order object
   * @param  $key string
   * @return none
   */
  function uc_post_back_call($data, $tid, $order, $key = NULL) {
    $payment_select   = $data['payment_method'];
    $key              = uc_novalnet_payment_key($payment_select);
    $paymentbank_enc  = array('novalnet_paypal', 'novalnet_instantbank', 'novalnet_ideal');
    $postback_data['key']            = $key[$payment_select];
    $postback_data['vendor']         = trim(variable_get($payment_select . '_vendorid', ''));
    $postback_data['auth_code']      = trim(variable_get($payment_select . '_authcode', ''));
    if (isset($data['payment_method'])) {
      if (in_array($data['payment_method'], $paymentbank_enc)) {
        $password =  trim(variable_get($payment_select . '_accesskey', ''));
        $postback_data['product']       = trim(uc_novalnet_decode_param($data['product'], $password));
        $postback_data['tariff']        = trim(uc_novalnet_decode_param($data['tariff'], $password));
      }
      else {
        $postback_data['product']       = trim((isset($data['product']) ? $data['product'] : $data['product_id']));
        $postback_data['tariff']        = trim($data['tariff']);
      }
    }
    $postback_data['order_no']    = trim($order->order_id);
    $postback_data['status']      = 100;
    $postback_data['tid']         = trim($tid);
    if ($order->payment_method == 'novalnet_prepayment' || $order->payment_method == 'novalnet_invoice') {
      $postback_data['invoice_ref'] = "BNR-" . $postback_data['product'] . "-" . $order->order_id;
    }
   if(!array_search('', $postback_data)) {
      $query = http_build_query($postback_data,  '',  '&');
      $options = array(
        'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
        'method' => 'POST',
        'data' => $query
        );
     $host      = uc_novalnet_redirect_urls('novalnet_postbackcall');
     $response  = drupal_http_request($host, $options);
     if(isset($_SESSION['nn'])) unset($_SESSION['nn']);
    }
  }
  /**
   * Perform post back call
   *
   * @param  $data array
   * @param  $tid double
   * @param  $order object
   * @param  $key string
   * @return none
   */
  function uc_validate_customer_name_mail(&$data) {
    if (!valid_email_address($data['email']) || (empty($data['first_name']) && empty($data['last_name']))) {
        $nnencodeerror       = t('Customer name/email fields are not valid');
        drupal_set_message(check_plain($nnencodeerror), 'warning');
        drupal_goto('cart/checkout');
        exit;
    }
    if (empty($data['first_name']) || empty($data['last_name']))  {
        $name = trim($data['first_name']) . trim($data['last_name']);
        list($data['first_name'], $data['last_name']) = preg_match('/\s/', $name) ? explode(' ', $name, 2) : array($name,$name);
    }
  }

function uc_novalnet_mail_alter(&$message) {    
    $message['headers']['Content-Type'] = 'text/html; charset=UTF-8; format=flowed';
    if ($message['id'] == 'uc_order_invoice' && $message['params']['order']->payment_method == 'novalnet_prepayment') {
    $message['body'][0] = preg_replace('|<div id=nn_prepayment_comments>(.+?)</div>|s',' ', $message['body'][0]);
    }
}
